#! /usr/bin/perl

my $rcs_id='RCS: $Id: faxrunqd.in,v 1.71 2006/01/13 13:58:40 gert Exp $';

use ExtUtils::testlib;
use yasdi;
use strict;
use POSIX;
use Getopt::Std;
use IO::Handle;
use vars qw($DSN $DBACCOUNT $DBPASSWORD $dbh $comm $childs %valuetypes $configfile $pidfile $yasdiini $protocolfile $currentdevice);

require "./yasdid.conf";

use vars qw / $opt_V $opt_c $opt_p $opt_d /;
$opt_V = 0;					# print version number
$opt_c = '';				# config file
$opt_p = '';				# protocol file
$opt_d = 1;
getopts( 'c:Vp:d:' ) ||
	die "Valid options: -c (config file), -p (protocol file), -d (device number), -V (version)\n";

if ($opt_c) {
	$configfile=$opt_c;
	debugprint (3, "using config file ".$configfile);
}

if ($opt_p) {
	$protocolfile=$opt_p;
	debugprint (3, "using protocol file ".$protocolfile);
}

if ($opt_d) {
	$currentdevice=$opt_d;
	debugprint (3, "using device number ".$currentdevice);
}

if ( $opt_V )		# print version info, and exit
{
    print <<EOF;

yasdi protocol to sql import by Stefan Helbing
$rcs_id

config read from '$configfile'

EOF
    exit 0;
}

#
# read config file
#
my %serials; #contains the assignment from serialnumber (key) to devicenumber (value)
my $pollinginterval;
my $protocoldata;
my %devicemonitoredchannels; #contains the assignment from devicenumber (key) to the names of the channels that should be observed (value)
my %devicemonitoredchannelhandles; #contains the assignment from devicenumber (key) to the handles of the channels that should be observed (value)
my $devicecount=0;
my $monchannelcount;
my $lastdaytotal=0;

if ( open( CF, $configfile ) ) {
    while( <CF> ) {
		next if /^\s*#/;		# comment lines
		chomp;
		next if /^\s*$/;		# empty lines

		if ( /^\[DEV(\d+)\]/ ) {
		} elsif ( /^\s*serial=(\S.*)/ ) {
		} elsif ( /^\s*protocol=(\S.*)/ ) {
			$protocoldata = $1;
		} elsif ( /^\s*channel=(\S.*)/ ) {
		} elsif ( /^\s*pollinginterval=(\d+)/ ) {
		} elsif ( /^\s*yasdiini=(\S.*)/ ) {
		} elsif ( /^\[global\]/ ) { #will be ignored
		} else {
			debugprint(1, "syntax error in $configfile, line ".$.."!");
			die "syntax error in $configfile, line ".$.."!\n";
		}
    }
    close (CF);
} else {
	die "Could not open config file ".$configfile.". Please specify a correct one with parameter -c!\n";
}

my @outputtitlearray;
if ( open( PF, $protocolfile ) ) {
	my $line=<PF>;
	chomp $line;
	@outputtitlearray=split /:/,$line;
} else {
	die "Could not open protocol file ".$protocolfile.". Please specify a correct one with parameter -p!\n";
}

my @protdata=split /:/,$protocoldata;

use DBI;
$DSN="dbi:mysql:database=".$protdata[2].";host=".$protdata[1];
$DBACCOUNT=$protdata[3];
$DBPASSWORD=$protdata[4];
# Verbindung zur DB
if ($dbh) {
	$dbh->disconnect;
}
$dbh = DBI->connect($DSN, $DBACCOUNT, $DBPASSWORD, { PrintError => 1 }) || die("Verbindung zu Datenbank $DSN fehlgeschlagen!");
$dbh->{AutoCommit}=1;
$dbh->{RaiseError}=1;
$dbh->{mysql_auto_reconnect}=1;

$comm = $dbh->prepare("SHOW TABLES LIKE '".$protdata[5]."_".$currentdevice."'");
$comm->execute;
if ($comm->rows==0) {
	my $createstring="CREATE TABLE `".$protdata[5]."_".$currentdevice."` (";
	for my $s (@outputtitlearray) {
		$createstring.="`".$s."` ".$valuetypes{$s}." NOT NULL,";
	}
	$createstring.="KEY `timestamp` (`timestamp`))";
	$comm = $dbh->prepare($createstring);
	$comm->execute;
}
#add missing columns
for my $s (@outputtitlearray) {
	$comm = $dbh->prepare("SHOW COLUMNS FROM `".$protdata[5]."_".$currentdevice."` LIKE '".$s."'");
	$comm->execute;
	if ($comm->rows==0) {
		$comm = $dbh->prepare("ALTER TABLE `".$protdata[5]."_".$currentdevice."` ADD `".$s."` ".$valuetypes{$s}." NOT NULL");
		$comm->execute;
	}
}
$comm = $dbh->prepare("SHOW TABLES LIKE '".$protdata[5]."_total'");
$comm->execute;
if ($comm->rows==0) {
	my $createstring="CREATE TABLE `".$protdata[5]."_total` (";
	$createstring.="`datum` DATE NOT NULL,";
	$createstring.="`device` TINYINT UNSIGNED NOT NULL,";
	$createstring.="`E-Total` FLOAT NOT NULL";
	$createstring.=");";
	$comm = $dbh->prepare($createstring);
	$comm->execute;
}

my $oldday="";
my $currentday="";
my $currentenergy=0.0;
my $oldenergy=0.0;
#Daten uebernehmen
while (<PF>) {
	my @fields=split /:/;
	$currentday=strftime("%Y-%m-%d", localtime($fields[0])); #first record contains timestamp
	if ($oldday eq "") { #this will be done for the first record only
		$oldday=$currentday;
		my $i=0;
		for my $s (@outputtitlearray) {
			if ($s eq "E-Total") {
				$oldenergy=$fields[$i];
			}
			$i++;
		}
	}
	if ($currentday ne $oldday) { #next day
		$comm = $dbh->prepare("SELECT datum FROM `".$protdata[5]."_total` WHERE datum='".$oldday."' AND device=".$currentdevice);
		$comm->execute;
		if ($comm->rows==0) { #if there doesn't exist a record for the day before we didn't make the summary
			#write the summary to database
			$comm = $dbh->prepare("INSERT INTO `".$protdata[5]."_total` SET `E-Total`=".($currentenergy-$oldenergy).", datum='".$oldday."', device=".$currentdevice);
			$comm->execute;
			$oldday=$currentday;
			$oldenergy=$currentenergy;
		} else {
			debugprint(2, "day ".$oldday." does already exist in SQL table ".$protdata[5]."_total. No summary will be written for that day.");
		}
	}
	my $insertstring="INSERT INTO `".$protdata[5]."_".$currentdevice."` SET ";
	for my $s (@outputtitlearray) {
		my $v=shift(@fields);
		$insertstring.="`".$s."`='".$v."',";
		if ($s eq "E-Total") {
			$currentenergy=$v;
		}
	}
	$insertstring=~s/,$//; #abschliessendes Komma entfernen
	$comm = $dbh->prepare($insertstring);
	$comm->execute;
}

close(PF);

if ($comm) {
	$comm->finish;
}
if ($dbh) {
	$dbh->disconnect;
}

#end

####################################################################################
#
# subroutines
#
#############

sub opendebuglog {
}

sub closedebuglog {
}

sub debugprint {
	my $debuglevel=shift;
	my $msg=shift;
	print strftime("%b %e %H:%M:%S",localtime())." ".$msg."\n";
}
