#! /usr/bin/perl

my $rcs_id='RCS: $Id: faxrunqd.in,v 1.71 2006/01/13 13:58:40 gert Exp $';

use ExtUtils::testlib;
use yasdi;
use strict;
use POSIX;
use Getopt::Std;
use vars qw($DSN $DBACCOUNT $DBPASSWORD $dbh $comm $childs %valuetypes);

my $configfile="/etc/yasdi/yasdid.ini";
my $pidfile="/var/run/yasdid.pid";
my $yasdiini="/etc/yasdi/yasdi.ini";
my $logfile="/var/log/yasdid.log";

my $debug=0;
my $daemonmode=0;

require "./yasdid.conf";

use vars qw / $opt_d $opt_V $opt_c $opt_l $opt_D /;
$opt_d = 0;					# debug
$opt_V = 0;					# print version number
$opt_D = 0;					# daemon
$opt_c = '';				# config file
$opt_l = '';				# log file
getopts( 'Ddc:Vl:' ) ||
	die "Valid options: -D (daemon), -d (debug), -c (config file), -l (log file), -V (version)\n";

if ($opt_d) {
	$debug=1;
}

if ($opt_l) {
	$logfile=$opt_l;
}

if ($opt_c) {
	$configfile=$opt_c;
	debugprint ("using config file ".$configfile);
}

if ( $opt_V )		# print version info, and exit
{
    print <<EOF;

yasdi daemon by Stefan Helbing
$rcs_id

config read from '$configfile'

EOF
    exit 0;
}

# fork as a daemon if invoked with -D
if ( $opt_D ) {
	debugprint("running in daemon mode");
	$daemonmode=1;
	my $pid = fork ;
	die "Cannot fork: $!" unless defined $pid ;
	if (0 == $pid) {
		# Child process: become leader of a new session, lose STDIN, log STDOUT/ERR
		POSIX::setsid();
		umask(022);
		open(STDIN, "/dev/null") ;
		open(STDOUT, ">&LOG") ;
		open(STDERR, ">&LOG") ;
	} else {
		exit 0;     # parent process
	}
}

$childs = 0;

#
# startup... write PID file, make sure no other daemon runs
#
if ( -f $pidfile && open( FP, $pidfile ) ) {
	my $p = <FP>; chomp $p; close FP;

	if ( $p ne '' && $p != $$ )	{ # does process exist?
		if ( kill( 0 => $p ) ||	$! == EPERM ) {
			die "yasdid: already running (PID=$p)\n";
		} else {			# no process found
		    unlink $pidfile;
		}
	}
}
open( FP, ">$pidfile" ) || die "yasdid: can't write PID to '$pidfile': $!\n";
print FP "$$\n";
close FP;

#
# set up handlers to handle "INT" (ctrl-c), "HUP" (hangup), "TERM" (kill)...
#    (handler function does cleanup, remove lock/pid files, etc., and exits)
#
$SIG{INT}  = \&signal_handler;			# cleanup & exit
$SIG{TERM} = \&signal_handler;			# cleanup & exit

$SIG{HUP}  = \&signal_handler_HUP;		# graceful exit
my $graceful_exit_requested = 0;

#
# read config file
#
my %serials; #contains the assignment from serialnumber (key) to devicenumber (value)
my $pollinginterval;
my $protocoldata;
my %devicemonitoredchannels; #contains the assignment from devicenumber (key) to the names of the channels that should be observed (value)
my %devicemonitoredchannelhandles; #contains the assignment from devicenumber (key) to the handles of the channels that should be observed (value)
my $currentdevice;
my $devicecount=0;
my $monchannelcount;
my $lastdaytotal=0;

if ( open( CF, $configfile ) ) {
    while( <CF> ) {
		print if $opt_d;

		next if /^\s*#/;		# comment lines
		chomp;
		next if /^\s*$/;		# empty lines

		if ( /^\[DEV(\d+)\]/ ) {
			$currentdevice = $1;
			$devicecount++;
			$monchannelcount=1;
		} elsif ( /^\s*serial=(\S.*)/ ) {
			if ($currentdevice eq "") {
				die "no device specified for line $.\n";
			} else {
				$serials{$1} = $currentdevice;
			}
		} elsif ( /^\s*protocol=(\S.*)/ ) {
			$protocoldata = $1;
		} elsif ( /^\s*channel=(\S.*)/ ) {
			if ($currentdevice eq "") {
				die "no device specified for line $.\n";
			} else {
				$devicemonitoredchannels{$currentdevice}{$1}=$monchannelcount++;
			}
		} elsif ( /^\s*pollinginterval=(\S.*)/ ) {
			$pollinginterval = $1;
		} elsif ( /^\s*yasdiini=(\S.*)/ ) {
			$yasdiini = $1;
		} elsif ( /^\[global\]/ ) {
		} else {
			die "syntax error in $configfile, line $.!\n";
		}
    }
    close (CF);
} else {
	die "Could not open config file ".$configfile.". Please specify a correct one with parameter -c!\n";
}

my $maxDriverIDs;
my $drivercount;
my $handlecount;
my %serialnumbers; #contains assignment from devicehandle (key) to serialnumber (value)
#my %channelhandles;
my $i;
my $channelname;
my $channelhandle;

debugprint("initalizing yasdi libraries...");
yasdi::yasdiMasterInitialize($yasdiini, $maxDriverIDs);

debugprint("getting yasdi driver id list...");
$drivercount=yasdi::pl_yasdiGetDriver($maxDriverIDs);
debugprint("found ".$drivercount." drivers");

debugprint("activating yasdi drivers...");
for($i=0;$i<$drivercount;$i++) {
	debugprint("using driver ".$i." with ID ".${yasdi::DriverIDArray}[$i]);
	my $name=yasdi::pl_yasdiGetDriverName(${yasdi::DriverIDArray}[$i]);
	my $result=yasdi::yasdiSetDriverOnline(${yasdi::DriverIDArray}[0]);
	if (!$result) {
		debugprint("Driver ".$name." could not be activated.");
		exit -1;
	} else {
		debugprint("Driver ".$name." has been activated.");
	}
}

$handlecount=1;
debugprint("found ".$handlecount." device handles");

debugprint("getting device names...");
my @channellogorder;
my @outputtitlearray;
my $outputfileexists=0;
my @protdata=split /:/,$protocoldata;
if ($protdata[0] eq "mysql") {
	use DBI;
	$DSN="dbi:mysql:database=".$protdata[2].";host=".$protdata[1];
	$DBACCOUNT=$protdata[3];
	$DBPASSWORD=$protdata[4];

	# Verbindung zur DB
	if ($dbh) {
		$dbh->disconnect;
	}
	$dbh = DBI->connect($DSN, $DBACCOUNT, $DBPASSWORD, { PrintError => 1 }) || harderror("Verbindung zu Datenbank $DSN fehlgeschlagen!");
	$dbh->{AutoCommit}=1;
	$dbh->{RaiseError}=1;
}
for($i=0;$i<$handlecount;$i++) {
		my $currentdevice=1;
		if ($protdata[0] eq "mysql") {
			if ($lastdaytotal==0) {
				#yesterday's mysql date string
				my $yesterday=strftime("%Y-%m-%d", 0, 0, 0, strftime("%d",localtime())-1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
				#did we already make a summary for yesterday?
				$comm = $dbh->prepare("SELECT datum FROM `".$protdata[5]."_total` WHERE datum='".$yesterday."'");
				$comm->execute;
				if ($comm->rows==0) { #if there doesn't exist a record for yesterday we didn't make the summary
					#today's mysql date string
					my $today=strftime("%Y-%m-%d", 0, 0, 0, strftime("%d",localtime()), strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
					#get first data record from yesterday
					$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<UNIX_TIMESTAMP('".$today."') ORDER BY timestamp ASC LIMIT 1");
					$comm->execute;
					my $firstrecord=$comm->fetchrow_array();
					#get last data record from yesterday
					$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<UNIX_TIMESTAMP('".$today."') ORDER BY timestamp DESC LIMIT 1");
					$comm->execute;
					my $lastrecord=$comm->fetchrow_array();
					$comm = $dbh->prepare("INSERT INTO `".$protdata[5]."_total` SET `E-Total`=".($lastrecord-$firstrecord).", datum='".$yesterday."', device=".$currentdevice);
					$comm->execute;
					$lastdaytotal=1;
				}
			}
		}
}

die; 
# Main loop
debugprint("starting main loop...");
while(1) {
	for($i=0;$i<$handlecount;$i++) { #for all devices
		my $handle=${yasdi::HandleArray}[$i];
		do_scan($handle);
	}

	my $sleep_time=$pollinginterval;
	sleep ( $sleep_time );
}

debugprint("deactivating yasdi drivers...");
for($i=0;$i<$drivercount;$i++) {
	my $name=yasdi::pl_yasdiGetDriverName(${yasdi::DriverIDArray}[$i]);
	my $result=yasdi::yasdiSetDriverOffline(${yasdi::DriverIDArray}[0]);
	if (!$result) {
		debugprint("Driver ".$name." could not be deactivated.");
	} else {
		debugprint("Driver ".$name." has been deactivated.");
	}
}

debugprint("shutting down yasdi...");
yasdi::yasdiMasterShutdown();

#end

####################################################################################
#
# subroutines
#
#############

sub do_scan {
	my $handle=shift;
	my $currentdevice=$serials{$serialnumbers{$handle}};
	debugprint("DEV".$currentdevice.": scanning device for new data");
	my @outputarray;
	my $dooutput=1; #standardmaessig Ausgabe vornehmen (ausser bei Fehler)
	$outputarray[0]=time();
	for (keys %{$devicemonitoredchannels{$currentdevice}}) { #run over all channelnames of a device
		$channelname=$_;
		$channelhandle=$devicemonitoredchannels{$currentdevice}{$channelname};
		my $value;
		my $valuetext;
		my $result=yasdi::pl_yasdiGetChannelValue($channelhandle,$handle,$value,$valuetext);
		if ($result==0) {
			$value=~s/^(\d+)\.(\d\d\d)(.*)/$1\.$2/; #auf die ersten drei Nachkommastellen begrenzen
			debugprint("DEV".$currentdevice.": Channel ".$channelname.": Value:".$value." ".$valuetext);
			$outputarray[$channellogorder[$channelhandle]]=$value;
		} else {
			if ($result==-1) {
				debugprint("DEV".$currentdevice.": invalid channelhandle for channel name ".$channelname);
			} elsif ($result==-2) {
				debugprint("DEV".$currentdevice.": yasdi driver is shutting down");
			} elsif ($result==-3) {
				debugprint("DEV".$currentdevice.": timeout getting a new value for channel name ".$channelname);
			} elsif ($result==-4) {
				debugprint("DEV".$currentdevice.": unknown error for channel name ".$channelname.". Invalid channel value.");
			} else {
				debugprint("DEV".$currentdevice.": other error(s) for channel name ".$channelname." (no answer, e.g.).");
			}
			$dooutput=0; #erfolglos: keine Ausgabe
		}
	}
	if ($dooutput==1) {
		if ($protdata[0] eq "file") {
			open(OUT,">>".$protdata[1]."_".$currentdevice) || print ("could not open protocol file ".$protdata[1]."_".$currentdevice." for append\n");
			print OUT join(":",@outputarray)."\n";
			close(OUT);
		}
		if ($protdata[0] eq "mysql") {
			my $insertstring="INSERT INTO `".$protdata[5]."_".$currentdevice."` SET ";
			for my $s (@outputtitlearray) {
				my $v=shift(@outputarray);
				$insertstring.="`".$s."`='".$v."',";
			}
			$insertstring=~s/,$//; #abschliessendes Komma entfernen
			$comm = $dbh->prepare($insertstring);
			$comm->execute;
		}
	}
}

sub debugprint {
	if ($debug) {
		my $msg=shift;
		if ($daemonmode==1) {
			open(LOG,">>".$logfile) || die("could not open ".$logfile." for write access");
			print LOG strftime("%b %e %H:%M:%S",localtime())." ".$msg."\n";
			close(LOG);
		} else {
			print $msg."\n";
		}
	}
}

##########################################################################
#
# signal_handler
#
# called before exit'ing, when user sent a HUP or INT signal...
#
##########################################################################
sub signal_handler {
	my $sig = shift;

	print "\nyasdid: signal handler: got signal $sig, goodbye...\n"
	if ($sig ne 'TERM' && $sig ne 'HUP' && $sig ne 'USR2');
	print LOG "\nyasdid: signal handler: got signal $sig, goodbye...\n";

	# remove PID file (-> global lock)
	print LOG "remove global lock $pidfile.\n" if $opt_d;
	unlink $pidfile;

	exit 7;
}

##########################################################################
#
# signal_handler_HUP
#
# called when user sends a HUP signal --> set flag to do graceful exit
#
##########################################################################
sub signal_handler_HUP {
	my $sig = shift;
	print LOG "\nyasdid: signal handler: got signal $sig, will exit as soon as possible...\n";
	$graceful_exit_requested = 1;
}
